// naumann@stce.rwth-aachen.de

#include <Eigen/Dense>
#include <iostream>
using namespace std;

template<typename T>
using VT=Eigen::VectorX<T>;

template<typename T>
using MT=Eigen::MatrixX<T>;

template<typename T>
T Householder(VT<T> x, VT<T> y) {
  VT<T> v=x; v(0)=v(0)-x(0)/fabs(x(0))*x.norm();
  VT<T> w=2*v/v.dot(v);
  x=x-w*v.dot(x);
  y=y-w*v.dot(y); 
  return y(0)/x(0);
}

int main() {
  int m=5; 
  using T=double;
  VT<T> x=VT<T>::Random(m), y=VT<T>::Random(m); 
  x=x.cwiseProduct(x);
  cout << "x^T=" << x.transpose() << endl << "y^T=" << y.transpose() << endl;
  T p=Householder(x,y);
  cout << "p=" << p << endl;
  return 0;
}
