// naumann@stce.rwth-aachen.de

#include "Eigen/Dense"
#include "dco.hpp"

#include <cmath>
using namespace std;

double eps=1e-7; // base accuracy of iterative methods

template<typename T>
T f(T p, T x) {
  return sin(p*x);
}

template<typename T>
T df_dx(T p, T x) {
  using TT=dco::gt1s<T>::type;
  TT xTT, yTT;
  xTT=x;
  dco::derivative(xTT)=1;
  yTT=f<TT>(p,xTT);
  return dco::derivative(yTT);
}

template<typename T>
T ddf_dx_dx(T p, T x) {
  using TT=dco::gt1s<T>::type;
  TT xTT, dfTT;
  xTT=x;
  dco::derivative(xTT)=1;
  dfTT=df_dx<TT>(p,xTT);
  return dco::derivative(dfTT);
}

template<typename T>
T dddf_dx_dx_dx(T p, T x) {
  using TT=dco::gt1s<T>::type;
  TT xTT, ddfTT;
  xTT=x;
  dco::derivative(xTT)=1;
  ddfTT=ddf_dx_dx<TT>(p,xTT);
  return dco::derivative(ddfTT);
}

template<typename T>
T newton(T p, T x) {
  T dfdx=df_dx(p,x);
  do {
    T ddfdxx=ddf_dx_dx(p,x);
    T dddfdxxx=dddf_dx_dx_dx(p,x);
    cout << x << ": " << fabs(dfdx*dddfdxxx/pow(ddfdxx,2)) << endl;
    x=x-dfdx/ddfdxx;
    dfdx=df_dx(p,x);
  } while (fabs(dfdx)>eps);
  return x;
}

int main() {
  using T=double;
  T p=1, x=0.1;
  cout << "y=" << f(p,x) << endl;
  x=newton(p,x);
  cout << "x=" << x << endl;
  cout << "y=" << f(p,x) << endl;
  cout << "dfdx=" << df_dx(p,x) << endl;
  cout << "ddfdxx=" << ddf_dx_dx(p,x) << endl;
  return 0;
}
